package org.dromara.controller;

import java.util.List;

import lombok.RequiredArgsConstructor;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.constraints.*;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.dromara.common.idempotent.annotation.RepeatSubmit;
import org.dromara.common.log.annotation.Log;
import org.dromara.common.web.core.BaseController;
import org.dromara.common.mybatis.core.page.PageQuery;
import org.dromara.common.core.domain.R;
import org.dromara.common.core.validate.AddGroup;
import org.dromara.common.core.validate.EditGroup;
import org.dromara.common.log.enums.BusinessType;
import org.dromara.common.excel.utils.ExcelUtil;
import org.dromara.domain.vo.TblBreakdownVo;
import org.dromara.domain.bo.TblBreakdownBo;
import org.dromara.service.ITblBreakdownService;
import org.dromara.common.mybatis.core.page.TableDataInfo;

/**
 * 故障管理
 *
 * @author Lion Li
 * @date 2024-08-11
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/system/breakdown")
public class TblBreakdownController extends BaseController {

    private final ITblBreakdownService tblBreakdownService;

    /**
     * 查询故障管理列表
     */
    @SaCheckPermission("system:breakdown:list")
    @GetMapping("/list")
    public TableDataInfo<TblBreakdownVo> list(TblBreakdownBo bo, PageQuery pageQuery) {
        return tblBreakdownService.queryPageList(bo, pageQuery);
    }

    /**
     * 导出故障管理列表
     */
    @SaCheckPermission("system:breakdown:export")
    @Log(title = "故障管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(TblBreakdownBo bo, HttpServletResponse response) {
        List<TblBreakdownVo> list = tblBreakdownService.queryList(bo);
        ExcelUtil.exportExcel(list, "故障管理", TblBreakdownVo.class, response);
    }

    /**
     * 获取故障管理详细信息
     *
     * @param id 主键
     */
    @SaCheckPermission("system:breakdown:query")
    @GetMapping("/{id}")
    public R<TblBreakdownVo> getInfo(@NotNull(message = "主键不能为空")
                                     @PathVariable Long id) {
        return R.ok(tblBreakdownService.queryById(id));
    }

    /**
     * 新增故障管理
     */
    @SaCheckPermission("system:breakdown:add")
    @Log(title = "故障管理", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping()
    public R<Void> add(@Validated(AddGroup.class) @RequestBody TblBreakdownBo bo) {
        return toAjax(tblBreakdownService.insertByBo(bo));
    }

    /**
     * 修改故障管理
     */
    @SaCheckPermission("system:breakdown:edit")
    @Log(title = "故障管理", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping()
    public R<Void> edit(@Validated(EditGroup.class) @RequestBody TblBreakdownBo bo) {
        return toAjax(tblBreakdownService.updateByBo(bo));
    }

    /**
     * 删除故障管理
     *
     * @param ids 主键串
     */
    @SaCheckPermission("system:breakdown:remove")
    @Log(title = "故障管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public R<Void> remove(@NotEmpty(message = "主键不能为空")
                          @PathVariable Long[] ids) {
        return toAjax(tblBreakdownService.deleteWithValidByIds(List.of(ids), true));
    }
}
