package org.dromara.controller;

import java.util.List;

import lombok.RequiredArgsConstructor;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.constraints.*;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.dromara.common.idempotent.annotation.RepeatSubmit;
import org.dromara.common.log.annotation.Log;
import org.dromara.common.web.core.BaseController;
import org.dromara.common.mybatis.core.page.PageQuery;
import org.dromara.common.core.domain.R;
import org.dromara.common.core.validate.AddGroup;
import org.dromara.common.core.validate.EditGroup;
import org.dromara.common.log.enums.BusinessType;
import org.dromara.common.excel.utils.ExcelUtil;
import org.dromara.domain.vo.TblSensorRecordVo;
import org.dromara.domain.bo.TblSensorRecordBo;
import org.dromara.service.ITblSensorRecordService;
import org.dromara.common.mybatis.core.page.TableDataInfo;

/**
 * 传感器实时数据
 *
 * @author Lion Li
 * @date 2024-08-11
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/system/sensorRecord")
public class TblSensorRecordController extends BaseController {

    private final ITblSensorRecordService tblSensorRecordService;

    /**
     * 查询传感器实时数据列表
     */
    @SaCheckPermission("system:sensorRecord:list")
    @GetMapping("/list")
    public TableDataInfo<TblSensorRecordVo> list(TblSensorRecordBo bo, PageQuery pageQuery) {
        return tblSensorRecordService.queryPageList(bo, pageQuery);
    }

    /**
     * 导出传感器实时数据列表
     */
    @SaCheckPermission("system:sensorRecord:export")
    @Log(title = "传感器实时数据", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(TblSensorRecordBo bo, HttpServletResponse response) {
        List<TblSensorRecordVo> list = tblSensorRecordService.queryList(bo);
        ExcelUtil.exportExcel(list, "传感器实时数据", TblSensorRecordVo.class, response);
    }

    /**
     * 获取传感器实时数据详细信息
     *
     * @param id 主键
     */
    @SaCheckPermission("system:sensorRecord:query")
    @GetMapping("/{id}")
    public R<TblSensorRecordVo> getInfo(@NotNull(message = "主键不能为空")
                                     @PathVariable Long id) {
        return R.ok(tblSensorRecordService.queryById(id));
    }

    /**
     * 新增传感器实时数据
     */
    @SaCheckPermission("system:sensorRecord:add")
    @Log(title = "传感器实时数据", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping()
    public R<Void> add(@Validated(AddGroup.class) @RequestBody TblSensorRecordBo bo) {
        return toAjax(tblSensorRecordService.insertByBo(bo));
    }

    /**
     * 修改传感器实时数据
     */
    @SaCheckPermission("system:sensorRecord:edit")
    @Log(title = "传感器实时数据", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping()
    public R<Void> edit(@Validated(EditGroup.class) @RequestBody TblSensorRecordBo bo) {
        return toAjax(tblSensorRecordService.updateByBo(bo));
    }

    /**
     * 删除传感器实时数据
     *
     * @param ids 主键串
     */
    @SaCheckPermission("system:sensorRecord:remove")
    @Log(title = "传感器实时数据", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public R<Void> remove(@NotEmpty(message = "主键不能为空")
                          @PathVariable Long[] ids) {
        return toAjax(tblSensorRecordService.deleteWithValidByIds(List.of(ids), true));
    }
}
