package org.dromara.controller;

import java.util.List;

import lombok.RequiredArgsConstructor;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.constraints.*;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.dromara.common.idempotent.annotation.RepeatSubmit;
import org.dromara.common.log.annotation.Log;
import org.dromara.common.web.core.BaseController;
import org.dromara.common.mybatis.core.page.PageQuery;
import org.dromara.common.core.domain.R;
import org.dromara.common.core.validate.AddGroup;
import org.dromara.common.core.validate.EditGroup;
import org.dromara.common.log.enums.BusinessType;
import org.dromara.common.excel.utils.ExcelUtil;
import org.dromara.domain.vo.TblWarnVo;
import org.dromara.domain.bo.TblWarnBo;
import org.dromara.service.ITblWarnService;
import org.dromara.common.mybatis.core.page.TableDataInfo;

/**
 * 告警管理
 *
 * @author Lion Li
 * @date 2024-08-11
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/system/warn")
public class TblWarnController extends BaseController {

    private final ITblWarnService tblWarnService;

    /**
     * 查询告警管理列表
     */
    @SaCheckPermission("system:warn:list")
    @GetMapping("/list")
    public TableDataInfo<TblWarnVo> list(TblWarnBo bo, PageQuery pageQuery) {
        return tblWarnService.queryPageList(bo, pageQuery);
    }

    /**
     * 导出告警管理列表
     */
    @SaCheckPermission("system:warn:export")
    @Log(title = "告警管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(TblWarnBo bo, HttpServletResponse response) {
        List<TblWarnVo> list = tblWarnService.queryList(bo);
        ExcelUtil.exportExcel(list, "告警管理", TblWarnVo.class, response);
    }

    /**
     * 获取告警管理详细信息
     *
     * @param id 主键
     */
    @SaCheckPermission("system:warn:query")
    @GetMapping("/{id}")
    public R<TblWarnVo> getInfo(@NotNull(message = "主键不能为空")
                                     @PathVariable Long id) {
        return R.ok(tblWarnService.queryById(id));
    }

    /**
     * 新增告警管理
     */
    @SaCheckPermission("system:warn:add")
    @Log(title = "告警管理", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping()
    public R<Void> add(@Validated(AddGroup.class) @RequestBody TblWarnBo bo) {
        return toAjax(tblWarnService.insertByBo(bo));
    }

    /**
     * 修改告警管理
     */
    @SaCheckPermission("system:warn:edit")
    @Log(title = "告警管理", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping()
    public R<Void> edit(@Validated(EditGroup.class) @RequestBody TblWarnBo bo) {
        return toAjax(tblWarnService.updateByBo(bo));
    }

    /**
     * 删除告警管理
     *
     * @param ids 主键串
     */
    @SaCheckPermission("system:warn:remove")
    @Log(title = "告警管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public R<Void> remove(@NotEmpty(message = "主键不能为空")
                          @PathVariable Long[] ids) {
        return toAjax(tblWarnService.deleteWithValidByIds(List.of(ids), true));
    }
}
