package org.dromara.service.impl;

import cn.hutool.core.date.DateUtil;
import org.dromara.common.core.utils.MapstructUtils;
import org.dromara.common.core.utils.StringUtils;
import org.dromara.common.mybatis.core.page.TableDataInfo;
import org.dromara.common.mybatis.core.page.PageQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import org.dromara.domain.TblBreakdown;
import org.springframework.stereotype.Service;
import org.dromara.domain.bo.TblWarnBo;
import org.dromara.domain.vo.TblWarnVo;
import org.dromara.domain.TblWarn;
import org.dromara.mapper.TblWarnMapper;
import org.dromara.service.ITblWarnService;

import java.util.List;
import java.util.Map;
import java.util.Collection;

/**
 * 告警管理Service业务层处理
 *
 * @author Lion Li
 * @date 2024-08-11
 */
@RequiredArgsConstructor
@Service
public class TblWarnServiceImpl implements ITblWarnService {

    private final TblWarnMapper baseMapper;

    /**
     * 查询告警管理
     *
     * @param id 主键
     * @return 告警管理
     */
    @Override
    public TblWarnVo queryById(Long id){
        return baseMapper.selectVoById(id);
    }

    /**
     * 分页查询告警管理列表
     *
     * @param bo        查询条件
     * @param pageQuery 分页参数
     * @return 告警管理分页列表
     */
    @Override
    public TableDataInfo<TblWarnVo> queryPageList(TblWarnBo bo, PageQuery pageQuery) {
        LambdaQueryWrapper<TblWarn> lqw = buildQueryWrapper(bo);
        Page<TblWarnVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        return TableDataInfo.build(result);
    }

    /**
     * 查询符合条件的告警管理列表
     *
     * @param bo 查询条件
     * @return 告警管理列表
     */
    @Override
    public List<TblWarnVo> queryList(TblWarnBo bo) {
        LambdaQueryWrapper<TblWarn> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }

    private LambdaQueryWrapper<TblWarn> buildQueryWrapper(TblWarnBo bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<TblWarn> lqw = Wrappers.lambdaQuery();
        lqw.like(StringUtils.isNotBlank(bo.getName()), TblWarn::getName, bo.getName());
        lqw.eq(StringUtils.isNotBlank(bo.getVal()), TblWarn::getVal, bo.getVal());
        lqw.eq(bo.getSensorId() != null, TblWarn::getSensorId, bo.getSensorId());
        lqw.like(StringUtils.isNotBlank(bo.getSensorName()), TblWarn::getSensorName, bo.getSensorName());
        lqw.eq(StringUtils.isNotBlank(bo.getContent()), TblWarn::getContent, bo.getContent());
        lqw.eq(StringUtils.isNotBlank(bo.getHandler()), TblWarn::getHandler, bo.getHandler());
        lqw.eq(StringUtils.isNotBlank(bo.getHandleWay()), TblWarn::getHandleWay, bo.getHandleWay());
        lqw.eq(StringUtils.isNotBlank(bo.getHandleContent()), TblWarn::getHandleContent, bo.getHandleContent());
        lqw.eq(bo.getHandleTime() != null, TblWarn::getHandleTime, bo.getHandleTime());
        lqw.eq(bo.getAlarmTime() != null, TblWarn::getAlarmTime, bo.getAlarmTime());
        lqw.eq(StringUtils.isNotBlank(bo.getStatus()), TblWarn::getStatus, bo.getStatus());
        if(bo.getParams()!=null && bo.getParams().get("startTime")!=null && bo.getParams().get("endTime")!=null){
            String start= bo.getParams().get("startTime").toString();
            String end= bo.getParams().get("endTime").toString();
            lqw.gt( TblWarn::getCreateTime, start);
            lqw.lt( TblWarn::getCreateTime, end);
        }
        return lqw;
    }

    /**
     * 新增告警管理
     *
     * @param bo 告警管理
     * @return 是否新增成功
     */
    @Override
    public Boolean insertByBo(TblWarnBo bo) {
        TblWarn add = MapstructUtils.convert(bo, TblWarn.class);
        validEntityBeforeSave(add);
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setId(add.getId());
        }
        return flag;
    }

    /**
     * 修改告警管理
     *
     * @param bo 告警管理
     * @return 是否修改成功
     */
    @Override
    public Boolean updateByBo(TblWarnBo bo) {
        TblWarn update = MapstructUtils.convert(bo, TblWarn.class);
        if(update.getHandler()!=null && update.getHandleTime()==null){
            update.setHandleTime(DateUtil.date());
            update.setStatus("1");
        }
        validEntityBeforeSave(update);
        return baseMapper.updateById(update) > 0;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(TblWarn entity){
        //TODO 做一些数据校验,如唯一约束
    }

    /**
     * 校验并批量删除告警管理信息
     *
     * @param ids     待删除的主键集合
     * @param isValid 是否进行有效性校验
     * @return 是否删除成功
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if(isValid){
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        return baseMapper.deleteByIds(ids) > 0;
    }
}
