package org.dromara.system.controller.system;

import java.util.List;

import lombok.RequiredArgsConstructor;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.constraints.*;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.dromara.common.idempotent.annotation.RepeatSubmit;
import org.dromara.common.log.annotation.Log;
import org.dromara.common.web.core.BaseController;
import org.dromara.common.mybatis.core.page.PageQuery;
import org.dromara.common.core.domain.R;
import org.dromara.common.core.validate.AddGroup;
import org.dromara.common.core.validate.EditGroup;
import org.dromara.common.log.enums.BusinessType;
import org.dromara.common.excel.utils.ExcelUtil;
import org.dromara.system.domain.vo.TblPageVo;
import org.dromara.system.domain.bo.TblPageBo;
import org.dromara.system.service.ITblPageService;
import org.dromara.common.mybatis.core.page.TableDataInfo;

/**
 * 页面
 *
 * @author Lion Li
 * @date 2024-07-26
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/system/page")
public class TblPageController extends BaseController {

    private final ITblPageService tblPageService;

    /**
     * 查询页面列表
     */
    @SaCheckPermission("system:page:list")
    @GetMapping("/list")
    public TableDataInfo<TblPageVo> list(TblPageBo bo, PageQuery pageQuery) {
        return tblPageService.queryPageList(bo, pageQuery);
    }

    /**
     * 导出页面列表
     */
    @SaCheckPermission("system:page:export")
    @Log(title = "页面", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(TblPageBo bo, HttpServletResponse response) {
        List<TblPageVo> list = tblPageService.queryList(bo);
        ExcelUtil.exportExcel(list, "页面", TblPageVo.class, response);
    }

    /**
     * 获取页面详细信息
     *
     * @param id 主键
     */
    @SaCheckPermission("system:page:query")
    @GetMapping("/{id}")
    public R<TblPageVo> getInfo(@NotNull(message = "主键不能为空")
                                     @PathVariable Long id) {
        return R.ok(tblPageService.queryById(id));
    }

    /**
     * 新增页面
     */
    @SaCheckPermission("system:page:add")
    @Log(title = "页面", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping()
    public R<Void> add(@Validated(AddGroup.class) @RequestBody TblPageBo bo) {
        return toAjax(tblPageService.insertByBo(bo));
    }

    /**
     * 修改页面
     */
    @SaCheckPermission("system:page:edit")
    @Log(title = "页面", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping()
    public R<Void> edit(@Validated(EditGroup.class) @RequestBody TblPageBo bo) {
        return toAjax(tblPageService.updateByBo(bo));
    }

    /**
     * 删除页面
     *
     * @param ids 主键串
     */
    @SaCheckPermission("system:page:remove")
    @Log(title = "页面", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public R<Void> remove(@NotEmpty(message = "主键不能为空")
                          @PathVariable Long[] ids) {
        return toAjax(tblPageService.deleteWithValidByIds(List.of(ids), true));
    }
}
