package org.dromara.system.service.impl;

import cn.hutool.core.codec.Base64;
import org.dromara.common.core.utils.MapstructUtils;
import org.dromara.common.core.utils.StringUtils;
import org.dromara.common.mybatis.core.page.TableDataInfo;
import org.dromara.common.mybatis.core.page.PageQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.dromara.system.domain.bo.TblPageBo;
import org.dromara.system.domain.vo.TblPageVo;
import org.dromara.system.domain.TblPage;
import org.dromara.system.mapper.TblPageMapper;
import org.dromara.system.service.ITblPageService;

import java.util.List;
import java.util.Map;
import java.util.Collection;

/**
 * 页面Service业务层处理
 *
 * @author Lion Li
 * @date 2024-07-26
 */
@RequiredArgsConstructor
@Service
public class TblPageServiceImpl implements ITblPageService {

    private final TblPageMapper baseMapper;

    /**
     * 查询页面
     *
     * @param id 主键
     * @return 页面
     */
    @Override
    public TblPageVo queryById(Long id){
        return baseMapper.selectVoById(id);
    }

    /**
     * 分页查询页面列表
     *
     * @param bo        查询条件
     * @param pageQuery 分页参数
     * @return 页面分页列表
     */
    @Override
    public TableDataInfo<TblPageVo> queryPageList(TblPageBo bo, PageQuery pageQuery) {
        LambdaQueryWrapper<TblPage> lqw = buildQueryWrapper(bo);
        Page<TblPageVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        return TableDataInfo.build(result);
    }

    /**
     * 查询符合条件的页面列表
     *
     * @param bo 查询条件
     * @return 页面列表
     */
    @Override
    public List<TblPageVo> queryList(TblPageBo bo) {
        LambdaQueryWrapper<TblPage> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }

    private LambdaQueryWrapper<TblPage> buildQueryWrapper(TblPageBo bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<TblPage> lqw = Wrappers.lambdaQuery();
        lqw.like(StringUtils.isNotBlank(bo.getName()), TblPage::getName, bo.getName());
        lqw.eq(StringUtils.isNotBlank(bo.getContent()), TblPage::getContent, bo.getContent());
        lqw.eq(StringUtils.isNotBlank(bo.getExt1()), TblPage::getExt1, bo.getExt1());
        lqw.eq(StringUtils.isNotBlank(bo.getExt2()), TblPage::getExt2, bo.getExt2());
        return lqw;
    }

    /**
     * 新增页面
     *
     * @param bo 页面
     * @return 是否新增成功
     */
    @Override
    public Boolean insertByBo(TblPageBo bo) {
        TblPage add = MapstructUtils.convert(bo, TblPage.class);
        validEntityBeforeSave(add);
        if(add.getContent()!=null){
            add.setContent(Base64.decodeStr(add.getContent()));
        }
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setId(add.getId());
        }
        return flag;
    }

    /**
     * 修改页面
     *
     * @param bo 页面
     * @return 是否修改成功
     */
    @Override
    public Boolean updateByBo(TblPageBo bo) {
        TblPage update = MapstructUtils.convert(bo, TblPage.class);
        if(update.getContent()!=null){
            update.setContent(Base64.decodeStr(update.getContent()));
        }
        validEntityBeforeSave(update);
        return baseMapper.updateById(update) > 0;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(TblPage entity){
        //TODO 做一些数据校验,如唯一约束
    }

    /**
     * 校验并批量删除页面信息
     *
     * @param ids     待删除的主键集合
     * @param isValid 是否进行有效性校验
     * @return 是否删除成功
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if(isValid){
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        return baseMapper.deleteByIds(ids) > 0;
    }
}
